from django.conf import settings
from django.core.paginator import EmptyPage, Paginator


def paginate(request, items, per_page=settings.DEFAULT_PER_PAGE,
             page_key='page'):
    """
    Paginate an iterable of items, pulling the page number from the request.

    Returns a tuple of ``(paginator, page)``.
    """
    paginator = Paginator(items, per_page)

    try:
        page_number = int(request.GET[page_key])
        page = paginator.page(page_number)
    except (ValueError, KeyError, EmptyPage):
        page = paginator.page(1)

    return paginator, page



"""
Tests for the Jinja2 tags included in Coop
"""
from wagtail.models import Page
from wagtail.test.utils import WagtailPageTestCase
from wagtail.test.utils.form_data import rich_text

from tests.app.models import TestPage


class TestPageModel(WagtailPageTestCase):
    def setUp(self):
        super().setUp()
        self.root_page = Page.objects.get(pk=1)

        self.page = self.root_page.add_child(
            instance=TestPage(title="Test page", body="<p>Hello, world!</p>")
        )
        self.login()

    def test_get_template(self):
        """
        Check that the custom ``get_template`` method returns sensible things
        """
        self.assertEqual(self.page.get_template(None), "layouts/app/test_page.html")

    def test_create_without_show_in_menus(self):
        """
        Check that a page can still be created without sending anything for
        ``show_in_menus``, which has been removed from coop sites in favour of
        a menu builder.
        """
        self.assertCanCreate(
            self.root_page,
            TestPage,
            {
                "title": "Creation test",
                "body": rich_text("<p>Hello, world!</p>"),
                "slug": "creation-test",
            },
        )



"""
Tests for the Jinja2 tags included in Coop
"""
from django.http.request import QueryDict
from django.template import engines
from django.test import TestCase

from coop.jinja2tags import core
from tests.app.models import TestPage


class TestJinja2Tags(TestCase):
    engine = engines['jinja2']

    def render(self, string, context):
        return self.engine.from_string(string).render(context)

    def test_model_classname(self):
        self.assertEqual(core.model_classname(TestPage), 'page-app-testpage')
        instance = TestPage(title='My page', body='<p>Hello, world!</p>')
        self.assertEqual(core.model_classname(instance), 'page-app-testpage')

    def test_qs(self):
        class request():
            GET = QueryDict('foo=bar')
        ctx = {'request': request}

        # No extra keys should return it unchanged
        self.assertEqual(core.qs(ctx), 'foo=bar')

        # Updating keys
        self.assertEqual(core.qs(ctx, foo='baz'), 'foo=baz')
        self.assertEqual(core.qs(ctx, foo=1), 'foo=1')
        self.assertEqual(core.qs(ctx, foo=None), '')
        self.assertEqual(core.qs(ctx, foo=None, baz='quux'), 'baz=quux')

        # Adding keys
        # Dict order is undefined, so using assertIn with both possibilities
        self.assertIn(core.qs(ctx, baz='quux'),
                      ['foo=bar&baz=quux', 'baz=quux&foo=bar'])
        self.assertEqual(core.qs(ctx, foo=None, baz='quux'), 'baz=quux')

        # Pass in a custom querydict
        self.assertEqual(core.qs({}, QueryDict('new=dict')), 'new=dict')

    def test_tel_filter(self):
        self.assertEqual(core.tel('0361234567'),
                         'tel:0361234567')
        self.assertEqual(core.tel('(03) 6123 4567'),
                         'tel:03-6123-4567')
        self.assertEqual(core.tel('+61 3 6123 4567'),
                         'tel:+61-3-6123-4567')
        self.assertEqual(core.tel('0407 000 111'),
                         'tel:0407-000-111')

    def test_br_filter(self):
        template = '{{ s|br }}'
        self.assertEqual(self.render(template, {'s': 'hello world'}),
                         'hello world')
        self.assertEqual(self.render(template, {'s': 'hello\nworld'}),
                         'hello<br>world')
        self.assertEqual(self.render(template, {'s': 'hello\n\nworld'}),
                         'hello<br><br>world')

    def test_p_filter(self):
        template = '{{ s|p }}'
        self.assertEqual(self.render(template, {'s': 'hello world'}),
                         '<p>hello world</p>')
        self.assertEqual(self.render(template, {'s': 'hello\nworld'}),
                         '<p>hello<br>world</p>')
        self.assertEqual(self.render(template, {'s': 'hello\n\nworld'}),
                         '<p>hello</p>\n\n<p>world</p>')



import sentry_sdk
from django.conf import settings
from mailchimp3 import MailChimp
from mailchimp3.mailchimpclient import MailChimpError

if hasattr(settings, "MAILCHIMP_API_KEY") and hasattr(settings, "MAILCHIMP_USERNAME"):
    client = MailChimp(settings.MAILCHIMP_API_KEY, settings.MAILCHIMP_USERNAME)
else:
    client = None


def subscribe_user(email, data={}, list_id=settings.MAILCHIMP_LIST_ID):
    if not client:
        sentry_sdk.capture_message("Mailchimp client improperly configured")
        return False
    try:
        email = email.lower()
        data.update(
            {
                "status_if_new": "subscribed",
                "email_address": email,
            }
        )
        client.lists.members.create_or_update(list_id, email, data)
        return True
    except MailChimpError as e:
        # Swallow the error but let us know
        sentry_sdk.capture_exception(e)
        return False



from django import forms


class CustomForm(forms.Form):
    def __init__(self, extra_fields, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields.update(extra_fields)
        self.extra_field_names = list(extra_fields.keys())



from django.db import models
from wagtail.fields import StreamField

from .blocks import FormStreamBlock
from .views import form_page_view


class ProccessFormMixin(object):
    def get_form_field(self):
        raise NotImplementedError()

    def get_subject(self):
        raise NotImplementedError()

    def get_email_to(self):
        raise NotImplementedError()

    def get_reply_to(self):
        # Usually static not content
        raise NotImplementedError()

    def after_send(self, request, form):
        # Triggered after succesful email
        pass

    def serve(self, request):
        return form_page_view(request, self)


class FormBuilderPageMixin(ProccessFormMixin, models.Model):
    form = StreamField(FormStreamBlock(), use_json_field=True)
    email_subject = models.CharField(max_length=255, help_text='Subject for the automated email')
    email_to = models.EmailField(help_text='Address automated emails are sent to')

    def get_form_field(self):
        return self.form

    def get_subject(self):
        return self.email_subject

    def get_email_to(self):
        return self.email_to

    class Meta:
        abstract = True



from django.urls import path

from .views import styleguide

urlpatterns = [
    path('', styleguide, name='styleguide', kwargs={'page': 'index'}),
    path('<str:page>/', styleguide, name='styleguide'),
]



from .attributes import Maybe
from .blocks import ListLengthBlockFactory, RichTextBlockFactory, StreamFieldFactory
from .fuzzy import FuzzyDocument, FuzzyImage, FuzzyPage, FuzzyParagraphs, FuzzyWords
from .inline import OrderableFactory

__all__ = [
    "FuzzyDocument",
    "FuzzyImage",
    "FuzzyPage",
    "FuzzyParagraphs",
    "FuzzyWords",
    "ListLengthBlockFactory",
    "Maybe",
    "OrderableFactory",
    "RichTextBlockFactory",
    "StreamFieldFactory",
]



import datetime
import json
import os
import pathlib
import random
from pathlib import Path

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.files.storage import default_storage
from django.core.serializers.json import DjangoJSONEncoder
from django.utils import lorem_ipsum
from django.utils.text import capfirst
from wagtail.documents.models import Document
from wagtail.images import get_image_model

Image = get_image_model()


n = random.randint


def rbool(prob=0.5):
    return random.random() < prob


def lpar(num):
    """``num`` random paragraphs of lorem ipsum."""
    return ''.join('<p>{0}</p>'.format(p)
                   for p in lorem_ipsum.paragraphs(num, False))


def lwords(num):
    """``num`` random lorem ipsum words."""
    return capfirst(lorem_ipsum.words(num, False))


def tel():
    """ random international format (aus) telephone number eg. +61 (0) 412 345 678"""
    return '+{0} ({1}) {2} {3} {4}'.format(
        n(10, 99), n(0, 9), n(100, 999), n(100, 999), n(100, 999))


def rdate(future=True):
    """
    A random date. Set ``future`` to True for dates in the future, False for
    dates in the past. Possible dates span up to five years from today.
    """
    now = datetime.date.today()
    delta = datetime.timedelta(days=n(0, 1825))
    if future:
        return now + delta
    else:
        return now - delta


def r_heading(level, num):
    return '<h{0}>{1}</h{0}>'.format(level, lwords(num))


def r_bold(num):
    return '<p><b>{0}</b></p>'.format(lwords(num))


def r_italic(num):
    return '<p><i>{0}</i></p>'.format(lwords(num))


def r_ordered_list(list_elements):
    list_elems = []
    for _ in range(list_elements):
        list_elems.append('<li>{0}</li>\n'.format(lwords(n(2, 4))))
    return '<ol>\n{0}</ol>'.format('\n'.join(map(str, list_elems)))


def r_unordered_list(list_elements):
    list_elems = []
    for _ in range(list_elements):
        list_elems.append('<li>{0}</li>\n'.format(lwords(n(2, 4))))
    return '<ul>\n{0}</ul>'.format('\n'.join(map(str, list_elems)))


def hr():
    return '<hr>'


def r_anchor(num):
    return ("<a href='http://example.com'>{0}</a>".format(lwords(num)))


def r_image():
    image = get_random_image()
    return ('<embed alt="Example image" embedtype="image" format="fullwidth" id="{0}"/>'.format(image.id))


def embed_video():
    return '<embed embedtype="media" url="{0}"/>'.format(get_random_video_url())


def rich_text_example():
    """Layout all possible elements that can be used in rich text fields."""
    return open(os.path.join(os.path.dirname(__file__), 'rich_text_example.html'), 'r').read()


def random_rich_text_example(num, embeds=False):
    """
    Layout some possible elements that can be used in rich text fields in a random fashion
    Takes one argument, the number of elements. Set embeds=True if you wish to include images
    and videos
    """
    funcs = [
        lambda: r_heading(n(2, 5), n(1, 3)),
        lambda: r_bold(n(3, 5)),
        lambda: r_italic(n(3, 5)),
        lambda: r_ordered_list(n(3, 5)),
        lambda: r_unordered_list(n(3, 5)),
        lambda: r_anchor(n(3, 5)),
        lambda: lpar(n(3, 5)),
        hr,
    ]
    if embeds:
        funcs.extend([r_image, embed_video])
    return '\n'.join([random.choice(funcs)() for _ in range(num)])


def json_dumps(x):
    """Dump a value to JSON using the Django JSON encoder"""
    return json.dumps(x, cls=DjangoJSONEncoder)


def shuf(l1):
    """Generate a shuffled copy of a list"""
    l2 = l1[:]
    random.shuffle(l2)
    return l2


class once(object):
    """A decorator that only calls the decorated function once"""
    has_run = False
    value = None
    fn = None

    def __init__(self, fn):
        self.fn = fn

    def __call__(self, *args, **kwargs):
        if self.has_run:
            return self.value

        self.value = self.fn(*args, **kwargs)
        self.has_run = True
        return self.value


def setup_images(test_img_dir=None):
    def _is_image(filename):
        extensions = ['.jpg', '.jpeg', '.png', '.gif', '.webp']
        return pathlib.Path(filename).suffix.lower() in extensions

    image_prefix = 'original_images/test-images'
    dest_dirname = os.path.join(settings.MEDIA_ROOT, image_prefix)

    if test_img_dir is None:
        HERE = os.path.join(os.path.dirname(__file__))
        test_img_dir = os.path.join(HERE, 'test_images')
    # Copy all images from test_img_dir to dest_dirname
    for filename in os.listdir(test_img_dir):
        image_src_path = os.path.join(test_img_dir, filename)
        image_dest_path = os.path.join(dest_dirname, filename)
        if not default_storage.exists(image_dest_path) and _is_image(filename):
            with open(image_src_path, 'rb') as f:
                default_storage.save(image_dest_path, f)
    # Create Image models for each image in dest_dirname
    for filename in os.listdir(dest_dirname):
        if _is_image(filename):
            image_path = os.path.join(dest_dirname, filename)
            Image.objects.create(title=filename, file=image_path)


def get_random_image(category: str = None) -> Image:
    images = Image.objects.order_by('?')
    if category:
        images = images.filter(tags__name__in=[category])
    return images.first()


def get_image(filename):
    return Image.objects.get(title=filename)


def setup_documents(test_document_dir=None):
    dest_dir = Path(settings.MEDIA_ROOT, 'documents')

    if test_document_dir is None:
        here = Path(__file__).parent.absolute()
        test_document_dir = here.joinpath('test_documents')
    for src_file in test_document_dir.iterdir():
        doc_dest_path = dest_dir.joinpath(src_file.name).as_posix()  # default_storage needs a string
        if not default_storage.exists(doc_dest_path) and src_file.is_file():
            with open(src_file.as_posix(), 'rb') as doc_file:
                default_storage.save(doc_dest_path, doc_file)
    for file in dest_dir.iterdir():
        if file.is_file():
            Document.objects.create(
                title=file.name,
                file=file
            )


def get_random_doc():
    return Document.objects.order_by('?').first()


def get_random_video_url():
    return random.choice([
        'https://youtu.be/H9VVkwRb_7M',
        'https://youtu.be/2WWwkArWajM',
        'https://youtu.be/yYaJWO_mNWs',
        'https://youtu.be/1kThvYlo2Lc',
        'https://youtu.be/Ji9qSuQapFY',
        'https://vimeo.com/155536745',
        'https://vimeo.com/179936131',
        'https://vimeo.com/188197250',
        'https://vimeo.com/175535505',
        'https://vimeo.com/188321226'
    ])


def make_superuser():
    User = get_user_model()
    if User.USERNAME_FIELD == 'email':
        User.objects.create_superuser(email='admin@admin.com', password='p')
    else:
        User.objects.create_superuser(username='admin', email='admin@admin.com', password='p')


def generate_form_streamfield(headings=True, file_field=True):
    """ Generates on field of each possible type """
    basic_types = ['text', 'textarea', 'email', 'number', 'date']

    if headings:
        yield ('heading', 'Basic Types')
    for field_type in basic_types:
        yield ('basic_field', {
            'label': field_type,
            'field_type': field_type,
            'help_text': lwords(10),
            'required': rbool(),
            'default_value': '',
            'placeholder': lwords(2),
        })

    if headings:
        yield ('heading', 'Choice Types')

    choice_types = ['select', 'checkbox', 'radio']

    for field_type in choice_types:
        yield ('choice_field', {
            'label': field_type,
            'field_type': field_type,
            'choices': [lwords(n(1, 2)) for _ in range(n(2, 8))],
            'help_text': lwords(10),
            'required': rbool(),
        })

    if file_field:
        if headings:
            yield ('heading', 'File types')
        yield ('file_field', {
            'label': 'File',
            'help_text': lwords(10),
            'required': rbool(),
        })



from django.test import TestCase
from wagtail.models import Page, Site

from coop.test import TestThingsWithContentMixin, TestThingsWithoutContentMixin
from tests.app.models import TestPage


class TestContentThings(TestThingsWithContentMixin, TestCase):
    def setUp(self):
        super().setUp()
        root_page = Page.objects.get(pk=1)
        self.home = root_page.add_child(instance=TestPage(
            title='Home',
            body='<p>Home</p>',
        ))
        self.site = Site.objects.create(
            root_page=self.home,
            hostname='localhost',
            is_default_site=True,
        )


class TestOtherThings(TestThingsWithoutContentMixin, TestCase):
    pass



from django.conf import settings
from django.template.loader import render_to_string
from django.utils.html import mark_safe
from wagtail import hooks
from wagtail.admin.ui.components import Component
from wagtail.models import Site
from wagtailcache.cache import clear_cache

from .models import AnalyticsSettings


class AnalyticsWarningPanel(Component):
    name = "analytics_warning_panel"
    order = 200

    def __init__(self, request):
        site = Site.find_for_request(request)
        if not site:
            raise ValueError("Need an active site to show analytics warnings")
        self.site = site
        super().__init__()

    def render_html(self, parent_context=None) -> str:
        instance = AnalyticsSettings.for_site(self.site)
        if not instance.google_analytics and not instance.google_tag_manager:
            return mark_safe(
                render_to_string(
                    "coop/analytics_warning.html",
                    {
                        "site": self.site,
                    },
                )
            )
        else:
            return ""


@hooks.register("construct_homepage_panels")
def analytics_warnings(request, panels):
    if settings.DEBUG:
        return
    try:
        analytics_panels = AnalyticsWarningPanel(request)
        panels.append(analytics_panels)
    except ValueError:
        pass


@hooks.register("after_create_page")
@hooks.register("after_edit_page")
def clear_wagtailcache(request, page):
    if page.live:
        clear_cache()



import json

import requests
from django import __version__ as django_version
from django.core.management.base import BaseCommand
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.text import slugify
from django.utils.timezone import now
from wagtail import __version__ as wt_version
from wagtail.models import Page, Site

from coop.models import AnalyticsSettings


class Command(BaseCommand):
    help = "Sends a JSON payload to the given url"

    def add_arguments(self, parser):
        parser.add_argument(
            "url",
            type=str,
            nargs="?",
            help="url to send the payload to",)

    def handle(self, *args, **options) -> None:
        url = options['url']

        def get_site_info(site):
            info = dict()
            analytics = AnalyticsSettings.for_site(site)

            if analytics:
                info['google_analytics'] = analytics.google_analytics
                info['google_tag_manager'] = analytics.google_tag_manager
            else:
                info['google_analytics'] = False
                info['google_tag_manager'] = False

            info['url'] = site.root_url
            info['name'] = site.site_name

            info['page_count'] = Page.objects.live().count() - 1  # Don't count the root page
            info['update_time'] = now().timestamp()
            info['wt_version'] = wt_version
            info['django_version'] = django_version

            return info

        payload = dict()
        for site in Site.objects.all():
            site_info = get_site_info(site)
            key = slugify(site.site_name)
            payload.update({key: site_info})

        requests.post(url, data=json.dumps(payload, cls=DjangoJSONEncoder))



import factory
from wagtail.blocks.stream_block import StreamValue
from wagtail_factories.blocks import StreamFieldFactory, StructBlockFactory

from coop.factories import FuzzyWords

from .blocks import ChoiceFieldBlock, FieldBlock, FileFieldBlock, FormBlocks


class BasicFieldBlockFactory(StructBlockFactory):
    label = FuzzyWords(3)
    help_text = FuzzyWords(5, True)
    required = factory.Faker('pybool')
    placeholder = FuzzyWords(3, True)

    class Meta:
        model = FieldBlock


class TextFieldBlockFactory(BasicFieldBlockFactory):
    field_type = 'text'


class TextAreaBlockFactory(BasicFieldBlockFactory):
    field_type = 'textarea'


class EmailFieldBlockFactory(BasicFieldBlockFactory):
    field_type = 'email'


class NumberFieldBlockFactory(BasicFieldBlockFactory):
    field_type = 'number'


class DateFieldBlockFactory(BasicFieldBlockFactory):
    field_type = 'date'


class TimeFieldBlockFactory(BasicFieldBlockFactory):
    field_type = 'time'


class CheckboxFieldFactory(BasicFieldBlockFactory):
    field_type = 'checkbox'


class BaseChoiceFactory(StructBlockFactory):
    label = FuzzyWords(3)
    help_text = FuzzyWords(5, True)
    required = factory.Faker('pybool')
    choices = factory.Faker('texts', nb_texts=6, max_nb_chars=10)

    class Meta:
        model = ChoiceFieldBlock


class SelectChoiceFactory(BaseChoiceFactory):
    field_type = 'select'


class CheckBoxChoiceFactory(BaseChoiceFactory):
    field_type = 'checkbox'


class RadioChoiceFactory(BaseChoiceFactory):
    field_type = 'radio'


class FileFieldFactory(StructBlockFactory):
    label = FuzzyWords(3)
    help_text = FuzzyWords(5, True)
    required = False

    class Meta:
        model = FileFieldBlock


class FormBlocksFactory(StreamFieldFactory):
    def __init__(self, **kwargs):
        factories = {}
        super().__init__(factories, **kwargs)

    def generate(self, instance, step):
        blocks = [
            ('basic_field', TextFieldBlockFactory()),
            ('basic_field', TextAreaBlockFactory()),
            ('basic_field', EmailFieldBlockFactory()),
            ('basic_field', NumberFieldBlockFactory()),
            ('basic_field', DateFieldBlockFactory()),
            ('basic_field', TimeFieldBlockFactory()),
            ('basic_field', CheckboxFieldFactory()),
            ('choice_field', SelectChoiceFactory()),
            ('choice_field', CheckBoxChoiceFactory()),
            ('choice_field', RadioChoiceFactory()),
            ('file_field', FileFieldFactory()),
            ('heading', 'Heading'),
        ]

        return StreamValue(FormBlocks(), blocks)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-13 05:30
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0028_merge'),
        ('coop', '0001_remove_site'),
    ]

    operations = [
        migrations.CreateModel(
            name='AnalyticsSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('google_analytics', models.CharField(blank=True, help_text='Your google analytics code', max_length=64, null=True)),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import posixpath

# Misc rubbish
SITE_ID = 1
DEFAULT_PER_PAGE = 20

# Sites built on coop all share a common URL structure and WSGI application
ROOT_URLCONF = "coop.urls"
WSGI_APPLICATION = "coop.wsgi.application"


# URLs for assets
ASSETS_URL = "/assets/"
MEDIA_URL = posixpath.join(ASSETS_URL, "media/")
STATIC_URL = posixpath.join(ASSETS_URL, "static/")


STATICFILES_STORAGE = "django.contrib.staticfiles.storage.StaticFilesStorage"


# What is installed by default
INSTALLED_APPS = [
    "coop",
    # Third party apps
    "taggit",
    "modelcluster",
    "wagtailmetadata",
    "wagtailcache",
    "django_htmx",
    "honeypot",
    # Django apps
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django.contrib.sitemaps.views",
    # Wagtail apps
    "wagtail",
    "wagtail.admin",
    "wagtail.documents",
    "wagtail.snippets",
    "wagtail.sites",
    "wagtail.users",
    "wagtail.images",
    "wagtail.embeds",
    "wagtail.search",
    "wagtail.contrib.redirects",
    "wagtail.contrib.settings",
    "wagtail.contrib.table_block",
]


try:
    import diskcache  # noqa

    INSTALLED_APPS.append("diskcache")

    DISK_CACHE = {
        "BACKEND": "diskcache.DjangoCache",
        "LOCATION": "/srv/cache",
        "TIMEOUT": 300,
        # ^-- Django setting for default timeout of each key.
        "SHARDS": 8,
        "DATABASE_TIMEOUT": 0.010,  # 10 milliseconds
        # ^-- Timeout for each DjangoCache database transaction.
        "OPTIONS": {"size_limit": 2**30},  # 1 gigabyte
    }
    CACHES = {"default": DISK_CACHE, "renditions": DISK_CACHE}
except ImportError:
    pass


# Localisation - most sites are Australian
TIME_ZONE = "Australia/Hobart"
LANGUAGE_CODE = "en-au"

USE_I18N = True
USE_L10N = False
USE_TZ = True


# Standard middleware required by Wagtail
MIDDLEWARE = [
    "wagtailcache.cache.UpdateCacheMiddleware",
    "django_htmx.middleware.HtmxMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
    "wagtailcache.cache.FetchFromCacheMiddleware",
]


# Wagtail uses Django templates, but we use Jinja templates.
# Some common Jinja extensions are used that cover most use-cases we need.
TEMPLATES_DJANGO = {
    "BACKEND": "django.template.backends.django.DjangoTemplates",
    "APP_DIRS": True,
    "OPTIONS": {
        "context_processors": [
            "django.contrib.auth.context_processors.auth",
            "django.template.context_processors.debug",
            "django.template.context_processors.i18n",
            "django.template.context_processors.media",
            "django.template.context_processors.static",
            "django.template.context_processors.tz",
            "django.contrib.messages.context_processors.messages",
            "django.template.context_processors.request",
            "wagtail.contrib.settings.context_processors.settings",
        ]
    },
}
TEMPLATES_JINJA = {
    "BACKEND": "django.template.backends.jinja2.Jinja2",
    "APP_DIRS": True,
    "OPTIONS": {
        "extensions": [
            "wagtail.jinja2tags.core",
            "wagtail.admin.jinja2tags.userbar",
            "wagtail.images.jinja2tags.images",
            "wagtail.contrib.settings.jinja2tags.settings",
            "coop.jinja2tags.core.Extension",
            "wagtailmetadata.jinja2tags.WagtailMetadataExtension",
        ],
    },
}

# Frontend bundler settings
try:
    import django_vite  # noqa

    # This is needed for django_vite to work, set to correct path in your project settings. e.g BASE_DIR.joinpath("src", "static")
    DJANGO_VITE_ASSETS_PATH = ""
    TEMPLATES_JINJA["OPTIONS"]["extensions"].append("coop.jinja2tags.vite.Extension")
    INSTALLED_APPS.append("django_vite")
except ImportError:
    pass

try:
    import webpack_loader  # noqa

    STATICFILES_STORAGE = (
        "django.contrib.staticfiles.storage.ManifestStaticFilesStorage"
    )
    TEMPLATES_JINJA["OPTIONS"]["extensions"].append("coop.jinja2tags.webpack.Extension")
    INSTALLED_APPS.append("webpack_loader")
except ImportError:
    pass


TEMPLATES = [TEMPLATES_DJANGO, TEMPLATES_JINJA]
WAGTAIL_CACHE = True


# Who to email when things break
ADMINS = [
    ("Django debug", "django@neonjungle.studio"),
]
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {
        "require_debug_false": {"()": "django.utils.log.RequireDebugFalse"},
        "bugsnag_filter": {
            "()": "coop.logging.RequireNoBugsnagSetting",
        },
        "sentry_filter": {
            "()": "coop.logging.RequireNoSentryInstalled",
        },
    },
    "handlers": {
        "mail_admins": {
            "level": "ERROR",
            "filters": ["require_debug_false", "bugsnag_filter", "sentry_filter"],
            "class": "django.utils.log.AdminEmailHandler",
        }
    },
    "loggers": {
        "django.request": {
            "handlers": ["mail_admins"],
            "level": "ERROR",
            "propagate": False,
        },
    },
}

# Stop us from getting moderation emails by default
WAGTAILADMIN_NOTIFICATION_INCLUDE_SUPERUSERS = False

# Max image upload size
WAGTAILIMAGES_MAX_UPLOAD_SIZE = 100 * 1024 * 1024  # 100mb

# Add the 'responsive-object' class to embeds by default
WAGTAILEMBEDS_RESPONSIVE_HTML = True

# Override if you actually need a zip_code
HONEYPOT_FIELD_NAME = "zip_code"

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"



import json
import re

import jinja2
import jinja2.ext
from django.conf import settings
from django.contrib.messages import get_messages
from django.contrib.staticfiles import finders
from django.contrib.staticfiles.storage import staticfiles_storage
from django.core.serializers.json import DjangoJSONEncoder
from django.template.defaultfilters import date, linebreaks_filter, linebreaksbr, time
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from markupsafe import Markup
from wagtail.models import Page, Site


def add_form_class(bound_field, new_classes):
    """Add a class to a django form field widget"""
    attrs = bound_field.field.widget.attrs
    classes = sorted(set(attrs.get("class", "").split() + new_classes.split()))
    attrs["class"] = " ".join(classes)
    return bound_field


def add_form_attributes(bound_field, attributes={}):
    """Add 1 or more attributes to a django form field widget"""
    bound_field.field.widget.attrs.update(attributes)
    return bound_field


def include_static(path):
    """
    Include the contents of a file that has been compiled into the static directory
    """
    fullpath = finders.find(path)
    if fullpath is None:
        raise ValueError("Could not find file %s" % path)

    with open(fullpath, "r") as f:
        return f.read()


def debug_variable(variable):
    """Print all the fields available on a variable"""
    return format_html("<pre>{}</pre>", dir(variable))


def url(name, *args, **kwargs):
    """Reverse a URL name. Alias of ``django.core.urlresolvers.reverse``"""
    return reverse(name, args=args or None, kwargs=kwargs or None)


@jinja2.pass_context
def site_root(context):
    """Get the root page for the site"""
    # This lookup will be cached on the intermediary objects, so this will only
    # hit the DB once per request
    request = context["request"]
    return Site.find_for_request(request).root_page.specific


def svg_inline(name):
    """
    Inline an SVG image. SVG images are expected to be a template, not kept in
    the static files
    """
    return mark_safe(render_to_string("svgs/%s.svg" % name))


@jinja2.pass_context
def breadcrumbs(context, page=None):
    """Print the top navigation menu for this site"""
    request = context.get("request")
    root = site_root(context)
    if page is None:
        page = context.get("page")
    ancestors = page.get_ancestors().filter(depth__gte=root.depth)

    return Markup(
        render_to_string(
            "tags/breadcrumbs.html",
            {
                "page": page,
                "ancestors": ancestors,
                "request": request,
            },
        )
    )


def model_classname(model_or_instance):
    """
    Generate a CSS class name from a Page model

    Usage::

        <html class="{{ self|model_classname }}">
    """
    if isinstance(model_or_instance, Page):
        model_or_instance = model_or_instance.content_type.model_class()

    try:
        meta = model_or_instance._meta
        return "page-{0}-{1}".format(meta.app_label, meta.model_name)
    except AttributeError:
        return ""


@jinja2.pass_context
def messages(context):
    """Get any messages from django.contrib.messages framework"""
    return get_messages(context.get("request"))


def json_dumps(value):
    """
    Dump the value to a JSON string. Useful when sending values to JavaScript
    """
    return Markup(json.dumps(value, cls=DjangoJSONEncoder))


@jinja2.pass_context
def qs(context, get=None, **kwargs):
    """
    Update a querydict with new values, and return it as a URL encoded string.
    Pass in the current ``request.GET``, and any items to set as keyword
    arguments. If a key has a value of ``None``, that key is removed from the
    querydict.

    The querydict is grabbed from ``request`.GET` in the context by default,
    but an alternative can be provided as the first positional argument.

    >>> request.GET.urlencode()
    "page=1&foo=bar"
    >>> qs(request.GET, page=2, baz="quux", foo=None)
    "page=2&baz=quux"
    """
    if get is None:
        get = context["request"].GET

    get = get.copy()
    for key, value in kwargs.items():
        if value is None:
            # Delete keys if value is None
            get.pop(key, None)
        else:
            get[key] = value
    return get.urlencode()


not_digit_re = re.compile(r"[^0-9+]+")


def tel(value):
    return "tel:{}".format(not_digit_re.sub("-", value).strip("-"))


def render_honeypot_field():
    return Markup(
        render_to_string(
            "honeypot/honeypot_field.html",
            {"fieldname": settings.HONEYPOT_FIELD_NAME, "value": ""},
        )
    )


@jinja2.pass_context
def analytics_js(context):
    hide: bool = settings.DEBUG
    if request := context.get("request"):
        hide |= getattr(request, "preview", False)
    umami_id = getattr(settings, "UMAMI_ID", None)
    if not hide and umami_id:
        return mark_safe(
            f'<script async src="https://stats.neonjungle.com.au/script.js" data-website-id="{umami_id}"></script>'
        )
    return ""


class Extension(jinja2.ext.Extension):
    def __init__(self, environment):
        super().__init__(environment)

        self.environment.globals.update(
            {
                "breadcrumbs": breadcrumbs,
                "site_root": site_root,
                "static": staticfiles_storage.url,
                "include_static": include_static,
                "url": url,
                "svg": svg_inline,
                "model_classname": model_classname,
                "messages": messages,
                "DEBUG": settings.DEBUG,
                "qs": qs,
                "render_honeypot_field": render_honeypot_field,
                "analytics_js": analytics_js,
            }
        )

        self.environment.filters.update(
            {
                "model_classname": model_classname,
                "add_form_class": add_form_class,
                "add_form_attributes": add_form_attributes,
                "debug_variable": debug_variable,
                "br": linebreaksbr,
                "p": linebreaks_filter,
                "json": json_dumps,
                "tel": tel,
                "time_format": time,
                "date_format": date,
            }
        )



import jinja2
import jinja2.ext
from webpack_loader.templatetags.webpack_loader import render_bundle as base_render_bundle


@jinja2.pass_context
def render_bundle(context, *args, **kwargs):
    return base_render_bundle(context, *args, **kwargs)


class Extension(jinja2.ext.Extension):
    def __init__(self, environment):
        super().__init__(environment)

        self.environment.globals.update({"render_bundle": render_bundle})



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-11 23:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop', '0003_tag_manager_settings'),
    ]

    operations = [
        migrations.AlterField(
            model_name='analyticssettings',
            name='google_tag_manager',
            field=models.CharField(blank=True, help_text='Your Google Tag Manager container ID, e.g GTM-XXXX', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='analyticssettings',
            name='google_analytics',
            field=models.CharField(blank=True, help_text='Your Google Analytics property ID, e.g UA-XXXXX-Y', max_length=255, null=True),
        ),
    ]



"""
Some testing tools that coop projects can use.

Most projects will want to do something like:

.. code-block:: python

    from coop.test import (
        TestThingsWithContentMixin, TestThingsWithoutContentMixin)
    from django.test import TestCase

    class TestContentThings(TestThingsWithContentMixin, TestCase):
        def setUp(self):
            super().setUp()
            root_page = Page.objects.get(pk=1)
            self.home = root_page.add_child(instance=HomePage(
                title='Home'))
            self.site = Site.objects.create(
                root_page=self.home,
                hostname='localhost',
                is_default_site=True)

    class TestOtherThings(TestThingsWithoutContentMixin, TestCase):
        pass
"""
from io import StringIO
from urllib.parse import urlsplit

from django.conf import settings
from django.contrib.staticfiles import finders
from django.core import mail
from django.core.management import call_command


class CoopAssertionMixin:
    standard_files = ["/favicon.ico", "/robots.txt", "/humans.txt"]

    def assertNoMissingMigrations(self):
        try:
            out = StringIO()
            call_command(
                *"makemigrations --check --dry-run --no-color --name=missing_migration".split(),
                stdout=out
            )
        except SystemExit:
            out.seek(0)
            self.fail(msg="Missing migrations:\n\n" + out.read())

    def assertTestdataWorks(self):
        """
        Run the ``testdata`` management command. The tests pass if no errors
        are thrown
        """
        call_command("testdata", stdout=StringIO())
        call_command("update_index", stdout=StringIO())

    def assertStandardFilesExist(self):
        for standard_file in self.standard_files:
            response = self.client.get(standard_file, follow=True)
            final_url = response.redirect_chain[-1][0]
            path = urlsplit(final_url).path
            self.assertTrue(path.startswith(settings.STATIC_URL))
            tail = path[len(settings.STATIC_URL):]
            self.assertTrue(finders.find(tail), msg="{} exists".format(path))

    def assert404PageRenders(self):
        url = "/quite/likely/not/a/page/"
        response = self.client.get(url)
        self.assertContains(response, url, status_code=404)

    def assert500PageRenders(self):
        response = self.client.get("/500/")
        self.assertEqual(response.status_code, 500)


class TestThingsWithContentMixin(CoopAssertionMixin):
    """
    Run all the standard coop assertions that require all the site content in
    the database.
    """

    def test_error_pages(self):
        """Test that the 404 and 500 pages render without error"""
        # 404 pages need database content to render site menus and stuff.
        self.assert404PageRenders()
        self.assert500PageRenders()

    def test_standard_files_exist(self):
        """Test that some standard files exist"""
        # This test will find the correct URL for the asset by following
        # redirects, which will also cause the 404 page to be rendered, so this
        # needs page content.
        self.assertStandardFilesExist()

    def test_cache_header_present(self):
        self.client.logout()
        response = self.client.get("/")
        self.assertTrue(response.has_header("X-Wagtail-Cache"))


class TestThingsWithoutContentMixin(CoopAssertionMixin):
    """
    Run all the standard coop assertions that do not require all the site
    content in the database.
    """

    def test_migrations(self):
        self.assertNoMissingMigrations()

    def test_testdata(self):
        self.assertTestdataWorks()


class TestContactPageMixin:
    """
    Use with standard BasicSiteTestCase, tests the email functionality.
    Override setUp with your ContactPageMixin subclass, assign it to self.contact_page
    """

    def get_form_data(self):
        """
        Based on the standard ContactForm, overrride if different
        """
        return {
            "name": "Example User",
            "email": "user@example.com",
            "phone_number": "0400 000 000",
            "message": "Hello, world!",
        }

    def test_send_email(self):
        data = self.get_form_data()

        response = self.client.post(self.contact_page.url, data)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(response.context_data["contact_form"].is_valid())
        self.assertTrue(response.context_data.get("success"))
        # Check email
        message = mail.outbox[0]
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(message.to, [self.contact_page.email_to])



from django.contrib.auth.models import AbstractUser, BaseUserManager
from django.contrib.postgres.fields import CIEmailField
from wagtail.search import index


class EmailUserManager(BaseUserManager):
    def _create_user(self, email, password, **extra_fields):
        """
        Create and save a user with the given username, email, and password.
        """
        if not email:
            raise ValueError("Email must be set")
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        extra_fields.setdefault("is_staff", False)
        extra_fields.setdefault("is_superuser", False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email, password=None, **extra_fields):
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_active", True)
        extra_fields.setdefault("is_superuser", True)
        return self._create_user(email, password, **extra_fields)


class AbstractEmailUser(index.Indexed, AbstractUser):
    username = None
    # FIXME this is deprecating in dj5
    email = CIEmailField("email address", unique=True)

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []

    objects = EmailUserManager()

    search_fields = [
        index.AutocompleteField("first_name"),
        index.SearchField("first_name"),
        index.AutocompleteField("last_name"),
        index.SearchField("last_name"),
        index.AutocompleteField("email"),
        index.SearchField("email"),
        index.FilterField("is_staff"),
        index.FilterField("is_superuser"),
    ]

    def __str__(self):
        if self.first_name and self.last_name:
            return f"{self.first_name} {self.last_name}"
        return self.email

    class Meta:
        abstract = True



from django import forms


class ContactForm(forms.Form):
    name = forms.CharField()
    email = forms.EmailField(required=False)
    phone_number = forms.CharField(required=False)
    message = forms.CharField(widget=forms.Textarea())



import factory
import faker
from factory.fuzzy import BaseFuzzyAttribute
from wagtail.documents import get_document_model
from wagtail.images import get_image_model
from wagtail.models import Page

from .utils import rbool

fake = faker.Faker("en_AU")
Image = get_image_model()
Document = get_document_model()


class FuzzyImage(BaseFuzzyAttribute):
    def fuzz(self) -> Image:
        return Image.objects.order_by("?").first()


class FuzzyPage(BaseFuzzyAttribute):
    def fuzz(self) -> Page:
        return Page.objects.order_by("?").first()


class FuzzyDocument(BaseFuzzyAttribute):
    def fuzz(self) -> Document:
        return Document.objects.order_by("?").first()


class FuzzyParagraphs(factory.Faker):
    def __init__(self, num=1, **kwargs):
        super().__init__("paragraphs", nb=num, **kwargs)

    def evaluate(self, instance, step, extra) -> str:
        value = super().evaluate(instance, step, extra)
        return "".join(f"<p>{p}</p>" for p in value)


class FuzzyWords(factory.Faker):
    def __init__(self, nb_words: int = 1, maybe: bool = False):
        self.maybe = maybe
        super().__init__("sentence", nb_words=nb_words)

    def evaluate(self, instance, step, extra) -> str:
        if self.maybe and not rbool():
            return ""
        # create a variable sentence, remove full stop
        value = super().evaluate(instance, step, extra)
        return value[:-1]



from django.forms.fields import BooleanField, MultipleChoiceField


def form_data_to_text(form, html=False):
    """ Helper method to turn a form response to a text (usually for email) """
    data_string = ""

    for bound_field in form:
        if isinstance(bound_field.field, BooleanField):
            value = form.cleaned_data[bound_field.name]
            value_display = 'Yes' if value else 'No'
            if html:
                data_string += f'<p>{bound_field.label}: {value_display}</p>'
            else:
                data_string += f'\n\n{bound_field.label}: {value_display}'
        elif isinstance(bound_field.field, MultipleChoiceField):
            values = form.cleaned_data[bound_field.name]
            if html:
                data_string += f'<p>{bound_field.label}:'
            else:
                data_string += f'\n\n{bound_field.label}:'
            for choice in bound_field.field.choices:
                value_display = 'Yes' if choice[0] in values else 'No'
                if html:
                    data_string += f'<br/>{choice[0]} - {value_display}'
                else:
                    data_string += f'\n{choice[0]} - {value_display}'
            if html:
                data_string += '</p>'
        else:
            value = form.cleaned_data[bound_field.name]
            if not value:
                value = 'Not filled out.'
            if html:
                data_string += f'<p>{bound_field.label}:<br/>{value}</p>'
            else:
                data_string += f'\n\n{bound_field.label}:\n{value}'

    return data_string



from typing import Any

import factory
import faker
from wagtail import blocks
from wagtail_factories.blocks import BlockFactory
from wagtail_factories.blocks import StreamFieldFactory as BaseStreamFieldFactory

fake = faker.Faker("en_AU")


class RichTextBlockFactory(BlockFactory):
    @classmethod
    def _build(cls, model_class, value=""):
        if not value:
            value = "".join(f"<p>{p}</p>" for p in fake.paragraphs(nb=3))
        block = model_class()
        return block.to_python(value)

    @classmethod
    def _create(cls, model_class, *args, **kwargs):
        return cls._build(model_class, *args, **kwargs)

    class Meta:
        model = blocks.RichTextBlock


class StreamFieldFactory(BaseStreamFieldFactory):
    """
    Extension of wagtail_factories.blocks.StreamFieldFactory that doesn't require passing parameters
    """

    def evaluate(self, instance, step, extra):
        if not extra:
            possible_blocks = self.stream_block_factory._meta.base_declarations.keys()
            extra = {
                f"{index}": block_name
                for index, block_name in enumerate(possible_blocks)
            }
        return super().evaluate(instance, step, extra)


class ListLengthBlockFactory(factory.SubFactory):
    """
    Returns an n list of generated blocks
    Usage: ListLengthBlockFactory(SubblockFactory, length=10, block_args=["content.ContentPage"])
    """

    def __init__(self, factory, length=1, block_args=None, **kwargs):
        self.length = length
        self.block_args = block_args or []
        super().__init__(factory, **kwargs)

    def __call__(self, *args: Any, **kwds: Any) -> Any:
        # TODO could be used with provided values then + (length - provided) generated appended
        return self.evaluate(None, None, kwds)

    def evaluate(self, instance, step, extra):
        subfactory = self.get_factory()
        ret_val = []
        for _ in range(self.length):
            # Very naive, assumes all default values
            ret_val.append(subfactory())

        list_block_def = blocks.list_block.ListBlock(
            subfactory._meta.model(*self.block_args)
        )
        return blocks.list_block.ListValue(list_block_def, ret_val)

    class Meta:
        model = blocks.ListBlock



"""
Remove the automatically generated site and home page
NOTE: This migration should be faked when converting a site to coop.
"""

from django.db import migrations


def remove_site(apps, schema):
    Site = apps.get_model('wagtailcore', 'site')
    Page = apps.get_model('wagtailcore', 'page')

    # Check if the site is premade
    site = Site.objects.first()
    if site.hostname == 'localhost':
        # Remove all the pre-defined sites
        Site.objects.all().delete()

        # Delete the home page
        Page.objects.get(pk=2).delete()
        # Fix the root page to show it has no children
        Page.objects.filter(pk=1).update(numchild=0)


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '__latest__'),
    ]

    operations = [
        migrations.RunPython(remove_site, migrations.RunPython.noop)
    ]



import factory

from .utils import rbool


class Maybe(factory.LazyAttribute):
    def evaluate(self, instance, step, extra):
        if rbool():
            lambda: super().evaluate(instance, step, extra)
        return None



from django.core.mail import EmailMessage
from django.forms.fields import FileField
from django.template.response import TemplateResponse
from honeypot.decorators import check_honeypot

from coop.utils.forms import form_data_to_text


@check_honeypot
def form_page_view(request, form_page, *args, **kwargs):
    form_block = None
    field_blocks = []
    if hasattr(form_page.get_form_field().stream_block, 'get_form'):
        # Form only streamfield
        form_block = form_page.get_form_field().stream_block
        field_blocks = form_page.get_form_field()
    else:
        # It's in a child block, let's go searching
        for child in form_page.get_form_field():
            if hasattr(child.block, 'get_form'):
                form_block = child.block
                field_blocks = [b for b in child.value]
                break

    context = form_page.get_context(request, *args, **kwargs)

    if request.method == 'POST' and form_block and field_blocks:
        form = form_block.get_form(field_blocks, data=request.POST, files=request.FILES)

        if form.is_valid():
            msg_content = '\nForm page title:\n{0}'.format(form_page.title)
            msg_content += '\n\nForm page URL:\n{0}'.format(form_page.full_url)
            msg_content += '\n\nSee form submission data below'
            msg_content += '\n------------------------------'
            msg_content += form_data_to_text(form)
            msg_content += '\n\n------------------------------'
            msg_content += '\nThis is an automated message, please do not reply.'

            msg = EmailMessage(
                form_page.get_subject(), msg_content,
                to=[form_page.get_email_to()],
                reply_to=[form_page.get_reply_to()]
            )

            for bound_field in form:
                name = bound_field.name
                if isinstance(bound_field.field, FileField) and name in request.FILES:
                    file = request.FILES[name]
                    msg.attach(file.name, file.read(), file.content_type)
            msg.send()

            form_page.after_send(request, form)
            context.update({
                'completed': True
            })
    elif form_block and field_blocks:
        form = form_block.get_form(field_blocks)
    else:
        form = None

    context.update({
        'form': form
    })
    return TemplateResponse(request, form_page.get_template(request, *args, **kwargs), context)



from wagtail.blocks.stream_block import StreamValue


def streamfield_mapper(model_name, field_name, mapper):
    def migration_function(apps, schema):
        SomeModel = apps.get_model(model_name)
        for instance in SomeModel.objects.all():
            data = getattr(instance, field_name).stream_data
            new_data = list(mapper(instance, data))
            if data != new_data:
                stream_block = getattr(instance, field_name).stream_block
                setattr(instance, field_name, StreamValue(stream_block, new_data, is_lazy=True))
                instance.save()

    return migration_function



from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from wagtail.models import Page, Site

from coop.models import AnalyticsSettings


class TestAnalyticsPanel(TestCase):
    def setUp(self):
        super().setUp()
        self.admin = User.objects.create_superuser(
            'admin', 'admin@example.com', 'p')
        self.client.login(username='admin', password='p')

    def make_link(self, site):
        url = reverse('wagtailsettings:edit', args=[
            'coop', 'analyticssettings', site.pk])
        return '<a href="{}">Add your relevant IDs</a>'.format(url)

    def test_no_site(self):
        """
        This used to throw an error if there was no Site, rendering the admin
        useless
        """
        response = self.client.get(reverse('wagtailadmin_home'))
        self.assertEqual(response.status_code, 200)

    def test_analytics_panel_appears(self):
        site = Site.objects.create(
            hostname='example.com',
            root_page=Page.objects.get(pk=1),
            is_default_site=True)

        response = self.client.get(reverse('wagtailadmin_home'))
        self.assertContains(response, self.make_link(site), html=True)

    def test_no_panel(self):
        site = Site.objects.create(
            hostname='example.com',
            root_page=Page.objects.get(pk=1),
            is_default_site=True)

        AnalyticsSettings.objects.create(
            site=site, google_analytics='UA-XXXX-XXXX')

        response = self.client.get(reverse('wagtailadmin_home'))
        self.assertNotContains(response, self.make_link(site), html=True)



from django.utils.safestring import mark_safe
from wagtail import blocks
from wagtail.documents.blocks import DocumentChooserBlock


class UnwrappedStreamBlock(blocks.StreamBlock):
    """Removes the surrounding divs around streamblocks."""

    def render_basic(self, value, context=None):
        return mark_safe('\n'.join(
            child.render(context=context) for child in value))


class LinkValue(blocks.StructValue):
    @property
    def href(self):
        if page := self.get('page'):
            return page.full_url
        elif doc := self.get('document'):
            return doc.url
        url = self.get('url', '')
        return url

    @property
    def text(self):
        if link_text := self.get('link_text'):
            return link_text
        elif page := self.get('page'):
            return page.title
        # Shouldn't get here
        return 'Click here'


class BaseLinkBlock(blocks.StructBlock):
    def render_basic(self, value: LinkValue, context=None):
        return mark_safe(
            f'<a href="{value.href}">{value.text}</a>')


class InternalLink(BaseLinkBlock):
    link_text = blocks.CharBlock(required=False, help_text='Leave blank to use page name')
    page = blocks.PageChooserBlock()

    class Meta:
        value_class = LinkValue
        icon = 'doc-empty'


class ExternalLink(BaseLinkBlock):
    link_text = blocks.CharBlock()
    url = blocks.URLBlock()

    class Meta:
        value_class = LinkValue
        icon = 'fa-external-link'


class DownloadLink(BaseLinkBlock):
    link_text = blocks.CharBlock(help_text='e.g. Download File')
    document = DocumentChooserBlock()

    class Meta:
        value_class = LinkValue
        icon = 'fa-download'


class LinkBlocks(UnwrappedStreamBlock):
    external = ExternalLink()
    internal = InternalLink()
    download = DownloadLink()

    class Meta:
        icon = 'fa-list'



import jinja2.ext
from django_vite.templatetags.django_vite import vite_asset, vite_hmr_client


class Extension(jinja2.ext.Extension):
    def __init__(self, environment):
        super().__init__(environment)

        self.environment.globals.update(
            {
                "vite_hmr_client": vite_hmr_client,
                "vite_asset": vite_asset,
            }
        )



#!/usr/bin/env python3
"""Install coop"""

from setuptools import find_packages, setup

with open("coop/_version.py", "r") as f:
    version = None
    exec(f.read())

with open("README.rst", "r") as f:
    readme = f.read()


install_requires = ["wagtail>=3.0"]

setup(
    name="coop",
    version=version,
    description="Standard base to build Wagtail sites from",
    long_description=readme,
    author="Neon Jungle",
    author_email="developers@neonjungle.studio",
    url="https://gitlab.com/neonjungle/coop",
    install_requires=[
        "wagtail~=5.2.0",
        "django>=3.2,<=4.2",
        "psycopg2-binary~=2.9.0",
        "pytz>=0",
        "Jinja2~=3.1.0",
        "django-htmx~=1.16.0",
        "django-honeypot~=1.0.0",
        "wagtail-metadata~=5.0.0",
        "wagtail-icomoon",
        "wagtail-cache~=2.3.0",
        "wagtail-factories~=4.1.0",
        "sentry-sdk",
    ],
    extras_require={
        "all": [
            "mailchimp3~=3.0.0",
            "diskcache~=5.2.0",
        ],
        "cache": ["diskcache~=5.2.0"],
        "mailchimp": ["mailchimp3~=3.0.0"],
        "vite": [
            "django-vite~=2.1.0",
        ],
        "webpack": [
            "django-webpack-loader~=1.8.0",
        ],
    },
    zip_safe=False,
    license="BSD License",
    packages=find_packages(exclude=["tests*"]),
    include_package_data=True,
    package_data={},
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Django",
        "License :: OSI Approved :: BSD License",
    ],
)



import logging
from logging import LogRecord

from django.conf import settings


class RequireNoBugsnagSetting(logging.Filter):
    """We want to turn off normal email error logging when buggsnag settings are present"""

    def filter(self, record):
        return not hasattr(settings, "BUGSNAG")


class RequireNoSentryInstalled(logging.Filter):
    def filter(self, record: LogRecord) -> bool:
        try:
            import sentry_sdk  # noqa

            return False
        except ImportError:
            return True



import re

from django.conf import settings
from django.core.mail import EmailMessage
from django.db import models
from django.template.response import TemplateResponse
from wagtail import models as wagtailmodels
from wagtail.admin.panels import FieldPanel, MultiFieldPanel
from wagtail.admin.widgets.slug import SlugInput
from wagtailmetadata.models import MetadataPageMixin

from .forms import ContactForm


class Page(MetadataPageMixin, wagtailmodels.Page):
    """
    A base page with social metadata and a more sensible default template.
    """

    # Disable the creation of the related field accessor on Page
    # This prevents conflicts with model names and field names
    page_ptr = models.OneToOneField(
        wagtailmodels.Page, parent_link=True, related_name="+", on_delete=models.CASCADE
    )

    # We override promote panels to remove show_in_menus
    promote_panels = [
        MultiFieldPanel(
            [
                FieldPanel("slug", widget=SlugInput),
                FieldPanel("seo_title"),
                FieldPanel("search_description"),
                FieldPanel("search_image"),  # From MetadataPageMixin
            ],
            "Common page configuration",
        ),
    ]

    def get_template(self, request, *args, **kwargs):
        """
        Templates are named like ``layouts/app/model.html``.
        """
        label = self._meta.label.split(".")
        app = label[0]
        model_name = "_".join(re.findall("^[a-z]+|[A-Z][^A-Z]*", label[1])).lower()
        return "layouts/{}/{}.html".format(app, model_name)

    def get_object_title(self):
        site = self.get_site()
        return f"{self.get_meta_title()} | {site.site_name}"

    class Meta:
        abstract = True


class ContactPageMixin(models.Model):
    contact_form_class = ContactForm

    email_to = models.EmailField(
        help_text="The email address enquiries should be sent to"
    )

    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        context.update(
            {
                "gmaps_key": getattr(settings, "GMAPS_KEY", ""),
            }
        )
        return context

    def send_email(self, form):
        data = form.cleaned_data
        msg_content = "\n".join(
            ["{0}: {1}".format(field.label, data[field.name]) for field in form]
        )
        msg = EmailMessage(
            "Website enquiry from %s" % data["name"],
            msg_content,
            to=[self.email_to],
            reply_to=[data["email"]],
        )
        msg.send()

    def serve(self, request):
        request.is_preview = getattr(request, "is_preview", False)
        context = self.get_context(request)
        ContactFormClass = self.contact_form_class

        if request.method == "POST":
            form = ContactFormClass(request.POST)
            if form.is_valid():
                self.send_email(form)
            context.update(
                {
                    "success": form.is_valid(),
                }
            )
        else:
            form = ContactFormClass()

        context.update({"contact_form": form})
        return TemplateResponse(request, self.get_template(request), context)

    class Meta:
        abstract = True



from django.db import models
from wagtail.contrib.settings.models import BaseSiteSetting, register_setting


@register_setting(icon='site')
class AnalyticsSettings(BaseSiteSetting):
    google_analytics = models.CharField(
        max_length=255, blank=True, null=True,
        help_text='Your Google Analytics property ID, e.g UA-XXXXX-Y')
    google_tag_manager = models.CharField(
        max_length=255, blank=True, null=True,
        help_text='Your Google Tag Manager container ID, e.g GTM-XXXX'
    )



from collections import OrderedDict

from django import forms
from django.utils.text import slugify
from wagtail import blocks

from coop.blocks import UnwrappedStreamBlock

from .forms import CustomForm


class BaseFieldBlock(blocks.StructBlock):
    def to_field(self, value):
        defaults = {'label': value['label'], 'required': value['required'],
                    'help_text': value['help_text']}
        field_class = self.get_field_class(value, defaults)
        return slugify(value['label']), field_class(**defaults)

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        field_details = self.to_field(value)
        context.update({
            'field_name': field_details[0],
            'field': field_details[1],
            'bound_field': parent_context['form'][slugify(value['label'])]
        })
        return context

    class Meta:
        abstract = True


class FieldBlock(BaseFieldBlock):
    label = blocks.CharBlock()
    field_type = blocks.ChoiceBlock(choices=[
        ('text', 'Single line text'),
        ('textarea', 'Multi line text'),
        ('email', 'Email'),
        ('number', 'Number'),
        ('date', 'Date'),
        ('time', 'Time'),
        ('checkbox', 'Checkbox'),
    ])
    help_text = blocks.CharBlock(required=False)
    required = blocks.BooleanBlock(default=True, required=False)
    placeholder = blocks.CharBlock(required=False)
    default_value = blocks.CharBlock(required=False)

    def get_field_class(self, value, defaults):
        defaults.update({'initial': value['default_value']})
        field_type = value['field_type']
        widget_props = {'attrs': {'placeholder': value['placeholder']}}
        if field_type == 'textarea':
            defaults.update({'widget': forms.Textarea(**widget_props)})
            return forms.CharField
        elif field_type == 'email':
            defaults.update({'widget': forms.EmailInput(**widget_props)})
            return forms.EmailField
        elif field_type == 'number':
            defaults.update({'widget': forms.NumberInput(**widget_props)})
            return forms.IntegerField
        elif field_type == 'date':
            widget_props['attrs']['type'] = 'date'
            defaults.update({'widget': forms.DateInput(**widget_props)})
            return forms.DateField
        elif field_type == 'time':
            widget_props['attrs']['type'] = 'time'
            defaults.update({'widget': forms.TimeInput(**widget_props)})
            return forms.TimeField
        elif field_type == 'checkbox':
            return forms.BooleanField
        # Default - assume text field
        defaults.update({
            'widget': forms.TextInput(**widget_props)
        })
        return forms.CharField

    class Meta:
        icon = 'fa-edit'
        template = 'blocks/form/form_field.html'


class ChoiceFieldBlock(BaseFieldBlock):
    label = blocks.CharBlock()
    field_type = blocks.ChoiceBlock(choices=[
        ('select', 'Dropdown'),
        ('checkbox', 'Checkboxes'),
        ('radio', 'Radio buttons'),
    ])
    choices = blocks.ListBlock(blocks.CharBlock(icon='fa-square-o'))
    help_text = blocks.CharBlock(required=False)
    required = blocks.BooleanBlock(default=True, required=False)

    def get_field_class(self, value, defaults):
        widget = {
            'select': forms.widgets.Select(),
            'checkbox': forms.widgets.CheckboxSelectMultiple(),
            'radio': forms.widgets.RadioSelect(),
        }[value['field_type']]
        defaults.update({
            'choices': [(c, c) for c in value['choices']],
            'widget': widget
        })
        if value['field_type'] == 'checkbox':
            return forms.MultipleChoiceField
        return forms.ChoiceField

    class Meta:
        icon = 'fa-check-square-o'
        template = 'blocks/form/form_field.html'


class FileFieldBlock(BaseFieldBlock):
    label = blocks.CharBlock()
    help_text = blocks.CharBlock(required=False)
    required = blocks.BooleanBlock(default=True, required=False)

    def get_field_class(self, value, defaults):
        return forms.FileField

    class Meta:
        icon = 'fa-file'
        template = 'blocks/form/file_upload.html'


class HeadingBlock(blocks.CharBlock):
    class Meta:
        template = 'blocks/form/form_heading.html'
        icon = 'fa-header'


class FormBlocks(UnwrappedStreamBlock):
    basic_field = FieldBlock()
    choice_field = ChoiceFieldBlock()
    file_field = FileFieldBlock()
    heading = HeadingBlock()

    def get_fields_from_blocks(self, blocks):
        for child in blocks:
            block = child.block
            if hasattr(block, 'to_field'):
                yield block.to_field(child.value)

    def get_form(self, blocks, **kwargs):
        extra_fields = OrderedDict(self.get_fields_from_blocks(blocks))
        return CustomForm(extra_fields, **kwargs)

    class Meta:
        icon = 'fa-pencil'


class FormStreamBlock(UnwrappedStreamBlock):
    form = FormBlocks()

    class Meta:
        block_counts = {
            'form': {
                'max_num': 1,
            }
        }
